SuitFlashlight = SuitFlashlight or {}

local mod_path = (SuitFlashlightCore and SuitFlashlightCore:GetPath()) or ModPath
SuitFlashlight._mod_core = SuitFlashlightCore --not currently needed for anything
SuitFlashlight._mod_path = mod_path
SuitFlashlight._assets_path = mod_path .. "assets/"
SuitFlashlight._menu_path = mod_path .. "options.json"
SuitFlashlight._loc_path = mod_path .. "loc/en.json"
SuitFlashlight._save_path = SavePath .. "suitflashlight.json"

SuitFlashlight._light_obj = nil
SuitFlashlight._light_effect = nil
SuitFlashlight._texture_normal = "units/lights/spot_light_projection_textures/spotprojection_11_flashlight_df"
SuitFlashlight._texture_af = "units/lights/goated/spotprojection_aprilfools_flashlight_df"
SuitFlashlight._range = 800 --max illumination distance; default on weapon flashlights is 1000 (10m)
SuitFlashlight._angle = 75 --illumination cone angle; default on weapon flashlights is 60 degrees
SuitFlashlight._light_multiplier = 0.9 --intensity multiplier (default 1)
SuitFlashlight._light_on = false

SuitFlashlight.settings = {
--no menu option for these
	a_offset_x = -20, -- +right/-left
	a_offset_y = 0, -- +forward/-backward
	a_offset_z = -35, -- +up/-down 
	a_offset_yaw = -1,
	a_offset_pitch = 90,
	a_offset_roll = 0,
	april_fools_mode = 1,
	sound_enabled = true,
	light_glow_enabled = false,
	
	light_color = "fff7dc",
	palettes = {
		"ff0000",
		"ffff00",
		"00ff00",
		"00ffff",
		"0000ff",
		"880000",
		"888800",
		"008800",
		"008888",
		"000088",
		"ff8800",
		"88ff00",
		"00ff88",
		"0088ff",
		"8800ff",
		"884400",
		"448800",
		"008844",
		"004488",
		"440088",
		"ffffff",
		"bbbbbb",
		"888888",
		"444444",
		"000000"
	}
}


--flashlight operation functions

function SuitFlashlight:AddFlashlight(state)
	local player = managers.player:local_player()
	if alive(player) then 
		local texture = self._texture_normal
		
		if self:IsAprilFoolsEnabled() then
			texture = self._texture_af
		end

		--create light object
		local light = World:create_light("spot|specular|plane_projection", texture)
		self._light_obj = light
		light:set_spot_angle_end(self._angle)
		light:set_far_range(self._range)
--		local a_obj = player:get_object(self._a_obj_name)
		local a_obj = player:camera():camera_object()
		if a_obj then
			light:link(a_obj)
			light:set_local_rotation(self:GetOffsetRot())
			light:set_local_position(self:GetOffsetPos())
			local color = Color(self.settings.light_color)
			light:set_color(Vector3(color.r,color.g,color.b))
			light:set_multiplier(self._light_multiplier)
			
			local light_effect_enabled = self.settings.light_glow_enabled
			if light_effect_enabled then 
				local light_effect = World:effect_manager():spawn({
					effect = Idstring("effects/particles/weapons/flashlight/fp_flashlight_multicolor"),
					parent = a_obj
				})
				if light_effect ~= -1 then
					self._light_effect = light_effect
					light_effect:set_rotation(self:GetOffsetRot())
					light_effect:set_position(self:GetOffsetPos())
				end
			end
		end
		
		if state ~= nil then
			light:set_enable(state)
		end
		
		return light
	end
end

function SuitFlashlight:RemoveFlashlight()
	self._light_on = nil
	
	if alive(self._light_obj) then 
		World:delete_light(self._light_obj)
	end
	self._light_obj = nil
	
	if self._light_effect ~= -1 and alive(self._light_effect) then
		World:effect_manager():kill(self._light_effect)
	end
	self._light_effect = nil
end

function SuitFlashlight:ToggleLight(force_state)
	local state = not self._light_on
	if force_state ~= nil then 
		state = force_state
	end
	self._light_on = state
	
	local player = managers.player:local_player()
	
	if not alive(player) then 
		self:RemoveFlashlight()
		return
	end
	
	local sound_enabled = self.settings.sound_enabled
	if sound_enabled then
		local sound_event = state and "gadget_flashlight_on" or "gadget_flashlight_off"
		player:sound_source():post_event(sound_event)
	end
	
	if alive(self._light_obj) then 
		self._light_obj:set_enable(state)
	else
		local light = self:AddFlashlight(state)
	end
	
	if alive(self._light_effect) then 
		World:effect_manager():set_hidden(self._light_effect, not state)
	end
	
end

function SuitFlashlight:SetLightColor(color)
	if alive(self._light_obj) then 
		self._light_obj:set_color(color)
	end
end

--menu and settings functions

function SuitFlashlight:GetOffsetPos()
	local settings = self.settings
	
	return Vector3(settings.a_offset_x,settings.a_offset_y,settings.a_offset_z)
end

function SuitFlashlight:GetOffsetRot()
	local settings = self.settings
	
	return Rotation(settings.a_offset_yaw,settings.a_offset_pitch,settings.a_offset_roll)
end

function SuitFlashlight:CheckAprilFools()
	local state = false
	
	local april_fools_mode = self.settings.april_fools_mode
	if april_fools_mode == 1 then
		local dato = os.date("%d/%m")
		if dato == "1/4" then
			state = true
		end
	elseif april_fools_mode == 2 then
		state = true
	end
	
	self.april_fools_mode_enabled = state
	return state
end

function SuitFlashlight:IsAprilFoolsEnabled()
	return self.april_fools_mode_enabled
end

function SuitFlashlight:GetPalettes()
	local palettes = {}
	for i,col_str in ipairs(self.settings.palettes) do 
		palettes[i] = Color(col_str)
	end
	return palettes
end

function SuitFlashlight:SetPalettes(palettes)
	for i,color in ipairs(palettes) do 
		self.settings.palettes[i] = ColorPicker.color_to_hex(color)
	end
end

function SuitFlashlight:callback_colorpicker_done(color,palettes,success)
	if success then 
		self.settings.light_color = ColorPicker.color_to_hex(color)
		self:SetLightColor(Color(color))
		self:SetPalettes(palettes)
		
		self:Save()
	end
end

function SuitFlashlight:Load()
	local file = io.open(self._save_path, "r")
	if (file) then
		for k, v in pairs(json.decode(file:read("*all"))) do
			self.settings[k] = v
		end
	else
		self:Save()
	end
end

function SuitFlashlight:Save()
	local file = io.open(self._save_path,"w+")
	if file then
		file:write(json.encode(self.settings))
		file:close()
	end
end

Hooks:Add( "MenuManagerInitialize", "MenuManagerInitialize_suitflashlight", function(menu_manager)
	SuitFlashlight:Load()
	
	if ColorPicker then
		SuitFlashlight._colorpicker = ColorPicker:new("suitflashlight",{
			color = Color(SuitFlashlight.settings.light_color),
			palettes = SuitFlashlight:GetPalettes(),
			changed_callback = nil,
			done_callback = callback(SuitFlashlight,SuitFlashlight,"callback_colorpicker_done")
		})
	end
	
	MenuCallbackHandler.callback_button_colorpicker_customize_toggle = function(self)
		if SuitFlashlight._colorpicker then 
			SuitFlashlight._colorpicker:Show({color = Color(SuitFlashlight.settings.light_color)})
		end
	end
	
	MenuCallbackHandler.callback_suitflashlight_unstuck = function(self)
		SuitFlashlight:RemoveFlashlight()
	end
	
	MenuCallbackHandler.callback_keybind_suitflashlight_toggle = function(self)
		SuitFlashlight:ToggleLight()
	end
	
	MenuHelper:LoadFromJsonFile(SuitFlashlight._menu_path, SuitFlashlight, SuitFlashlight.settings)
end)

Hooks:Add("LocalizationManagerPostInit", "LocalizationManagerPostInit_suitflashlight", function( loc )
	if not BeardLib then
		loc:load_localization_file(SuitFlashlight._loc_path)
	end
end)

Hooks:Add("BaseNetworkSessionOnLoadComplete","suitflashlight_ongameloaded",function()
	if SuitFlashlight:CheckAprilFools() then
		local type_ids = Idstring("texture")
		BLT.AssetManager:CreateEntry(Idstring(SuitFlashlight._texture_af),type_ids,SuitFlashlight._assets_path .. SuitFlashlight._texture_af .. ".texture")
		managers.dyn_resource:load(type_ids, Idstring(SuitFlashlight._texture_af), DynamicResourceManager.DYN_RESOURCES_PACKAGE, nil)
	end
end)